import {createActionHandlerDecorator} from "./index";

/**
 * If specified, the action cannot be entered by other sagas during execution.
 * For error handler action, mutex logic is auto added.
 */
export function Mutex() {
    const lockTimeMap: {[action: string]: number} = {};
    return createActionHandlerDecorator(function* (handler, thisModule) {
        const actionName = handler.actionName;
        if (lockTimeMap[actionName]) {
            thisModule.logger.info(actionName, {
                actionPayload: handler.maskedParams,
                mutexLocked: "true",
                lockedDuration: (Date.now() - lockTimeMap[actionName]).toString(),
            });
        } else {
            try {
                lockTimeMap[actionName] = Date.now();
                yield* handler();
            } finally {
                delete lockTimeMap[actionName];
            }
        }
    });
}
